Package: cobalt
Title: Covariate Balance Tables and Plots
Version: 4.6.3
Authors@R: c(
    person("Noah", "Greifer", role = c("aut", "cre"), 
           email = "noah.greifer@gmail.com",
	         comment = c(ORCID = "0000-0003-3067-7154"))
    )
Description: Generate balance tables and plots for covariates of groups preprocessed through 
             matching, weighting or subclassification, for example, using propensity scores. Includes 
             integration with 'MatchIt', 'WeightIt', 'MatchThem', 'twang', 'Matching', 'optmatch', 'CBPS', 'ebal', 
             'cem', 'sbw', and 'designmatch' for assessing balance on the output of their preprocessing 
             functions. Users can also specify data for balance assessment not generated through 
             the above packages. Also included are methods for assessing balance in clustered or 
             multiply imputed data sets or data sets with multi-category, continuous, or longitudinal treatments.
Depends: R (>= 4.1.0)
Imports: stats, utils, grid, grDevices, ggplot2 (>= 4.0.0), gtable (>=
        0.3.6), gridExtra (>= 2.3), arg (>= 0.1.1), rlang (>= 1.2.0),
        cli (>= 3.6.5)
Suggests: MatchIt (>= 4.0.0), WeightIt (>= 1.0.0), twang (>= 1.6),
        twangContinuous, Matching, optmatch, ebal, CBPS (>= 0.17),
        designmatch, optweight, mice (>= 3.8.0), MatchThem (>= 0.9.3),
        cem (>= 1.1.30), sbw (>= 1.1.5), gbm (>= 2.1.7), brglm2 (>=
        0.9), caret, knitr, rmarkdown, testthat (>= 3.0.0)
License: GPL (>= 2)
Encoding: UTF-8
LazyData: true
VignetteBuilder: knitr
URL: https://ngreifer.github.io/cobalt/,
        https://github.com/ngreifer/cobalt
BugReports: https://github.com/ngreifer/cobalt/issues
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-29 20:48:24 UTC; NoahGreifer
Author: Noah Greifer [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-3067-7154>)
Maintainer: Noah Greifer <noah.greifer@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-30 05:13:24 UTC
Built: R 4.5.2; ; 2026-05-30 08:41:19 UTC; unix
