# Generated by roxygen2: do not edit by hand

S3method(autoplot,bal.tab)
S3method(bal.compute,bal.init)
S3method(bal.compute,default)
S3method(bal.tab,CBMSM)
S3method(bal.tab,CBPS)
S3method(bal.tab,Match)
S3method(bal.tab,cem.match)
S3method(bal.tab,data.frame)
S3method(bal.tab,data.frame.list)
S3method(bal.tab,default)
S3method(bal.tab,designmatch)
S3method(bal.tab,ebalance)
S3method(bal.tab,formula)
S3method(bal.tab,formula.list)
S3method(bal.tab,iptw)
S3method(bal.tab,matchit)
S3method(bal.tab,matrix)
S3method(bal.tab,mimids)
S3method(bal.tab,mnps)
S3method(bal.tab,optmatch)
S3method(bal.tab,ps)
S3method(bal.tab,ps.cont)
S3method(bal.tab,sbwcau)
S3method(bal.tab,weightit)
S3method(bal.tab,wimids)
S3method(get.w,CBMSM)
S3method(get.w,CBPS)
S3method(get.w,Match)
S3method(get.w,cem.match)
S3method(get.w,designmatch)
S3method(get.w,ebalance)
S3method(get.w,iptw)
S3method(get.w,matchit)
S3method(get.w,mimids)
S3method(get.w,mnps)
S3method(get.w,optmatch)
S3method(get.w,ps)
S3method(get.w,ps.cont)
S3method(get.w,sbwcau)
S3method(get.w,weightit)
S3method(get.w,wimids)
S3method(plot,bal.tab)
S3method(print,bal.init)
S3method(print,bal.tab)
S3method(print,love.plot)
export(available.stats)
export(bal.compute)
export(bal.init)
export(bal.plot)
export(bal.tab)
export(col_w_corr)
export(col_w_cov)
export(col_w_dcorr)
export(col_w_dcov)
export(col_w_ks)
export(col_w_mean)
export(col_w_ovl)
export(col_w_sd)
export(col_w_smd)
export(col_w_vr)
export(f.build)
export(get.cobalt.options)
export(get.w)
export(love.plot)
export(set.cobalt.options)
export(splitfactor)
export(unsplitfactor)
export(var.names)
import(stats)
importFrom(ggplot2,.data)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
